/*
 * Juego.java
 *
 * Created on 13 de octubre de 2006, 03:17 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package clases;

import java.io.Serializable;
import java.util.Vector;

/**
 * Contiene todos los metodos necesarios para jugar al Triqui.
 * Maneja el Juego, el tablero de juego y los jugadores.
 *
 * @author Alejandro Muoz Andrade - almunoz
 */
public class Juego implements Serializable{
    public int numero;
    public boolean iniciado;
    public boolean terminado;
    public boolean esperando;
    public Tablero tablero;
    public Vector<Jugador> jugadores;
            
    /**
     * Crea una nueva instancia de Juego
     */
    public Juego() {
        numero = -1;
        iniciado = false;
        terminado = false;
        esperando = false;
        tablero = new Tablero();
        jugadores = new Vector<Jugador>(2, 1);
        jugadores.add(new Jugador(1, "X", true, numero));
        jugadores.add(new Jugador(2, "O", false, numero));
    }
    
    /**
     * Crea una nueva instancia de Juego.
     * @param num Numero del juego
     */
    public Juego(int num){
        numero = num;
        iniciado = false;
        terminado = false;
        esperando = false;
        tablero = new Tablero();
        jugadores = new Vector<Jugador>(2, 1);
        jugadores.add(new Jugador(1, "X", true, numero));
        jugadores.add(new Jugador(2, "O", false, numero));
    }
    
    /**
     * Reinicia el Juego a sus condiciones iniciales
     */
    public void reiniciar(){
        //numero = num;
        iniciado = false;
        terminado = false;
        esperando = false;
        tablero.reiniciar();
        jugadores.get(0).reiniciar(1, "X", true);
        jugadores.get(1).reiniciar(2, "O", false);
    }
    
    /**
     * Comprueba si el juego esta disponible
     * @return <code>true</code> - Si el juego esta disponible.<br>
     * <code>false</code> - Si el juego no esta disponible. 
     */
    public boolean estaDisponible(){
        if(iniciado==false)
            return true;
        else
            return false;
    }
    
    /**
     * Comprueba si el juego esta en espera de algun jugador
     * @return <code>true</code> - Si el juego esta esperando.<br>
     * <code>false</code> - Si el juego no esta esperando. 
     */
    public boolean estaEsperando(){
        if(esperando==true)
            return true;
        else
            return false;
    }
    
    /**
     * Inicia un juego nuevo y devuelve el jugador
     * @return El objeto jugador que fue asignado
     */
    public Jugador iniciarNuevo(){
        esperando = true;
        // Retorna el jugador que no esta asignado
        if(jugadores.get(0).asignado==false){
            jugadores.get(0).asignado = true;
            return jugadores.get(0);
        }
        else if(jugadores.get(1).asignado==false){
            jugadores.get(1).asignado = true;
            return jugadores.get(1);
        }
        return null;
    }
    
    /**
     * Busca un juego que ya se haya iniciado y devuelve el jugador
     * que no este asignado
     * @return El objeto jugador que fue asignado
     */
    public Jugador unirse(){
        esperando = false;
        iniciado = true;
        // Retorna el jugador que no esta asignado
        if(jugadores.get(0).asignado==false){
            jugadores.get(0).asignado = true;
            return jugadores.get(0);
        }
        else if(jugadores.get(1).asignado==false){
            jugadores.get(1).asignado = true;
            return jugadores.get(1);
        }
        return null;
    }
    
    /**
     * Intercambia los turnos de los jugadores
     */    
    public void intercambiarTurnos(){
        if(jugadores.get(0).turno==true){
            jugadores.get(0).turno = false;
            jugadores.get(1).turno = true;
        } else{
            jugadores.get(0).turno = true;
            jugadores.get(1).turno = false;
        }
    }
    
    /**
     * Comprueba si la posicin dada est vacia.
     * @param posicion La posicion que va ser comprobada en el tablero de juego.
     * Debe ser un nmero entre 1 y 9 
     * @return <code>true</code> - Si la posicin est vacia.
     * @return <code>false</code> - Si la posicin est ocupada o no es una posicin vlida.
     * @see Tablero#posicionVacia(int posicion)
     */
    public boolean posicionVacia(int posicion){
        return tablero.posicionVacia(posicion);
    }
    
    /**
     * Coloca una ficha en el tablero de juego
     * @param jugador El jugador que va a colocar la ficha
     * @param posicion La posicion de la ficha en el tablero (Entre 1 y 9)
     * @see Tablero#colocarFicha(Jugador jugador, int posicion)
     */
    public void colocarFicha(Jugador jugador, int posicion){
        tablero.colocarFicha(jugador, posicion);
    }
    
    /**
     * Conprueba si el juego se termino
     * @return <code>true</code> - Si el juego se termino.
     * @return <code>false</code> - Si el juego no se ha terminado.
     * @see Tablero#comprobarTerminado(int num)
     */
    public boolean comprobarTerminado(){
        boolean flag = false;
        if(terminado==true || tablero.posicionesLibres<=0){
            return true;
        }
        
        for (int num=1; num<=2; num++){
            if(tablero.comprobarTerminado(num)==true){
                return true;
            }
        }
        return false;
    }
    
    /**
     * Asigna valores a las variables <code>iniciado</code> y <code>terminado</code>
     * dependiendo si el juego ya se termino o no.
     * @see Juego#comprobarTerminado()
     */
    public void comprobar(){
        if(comprobarTerminado()){
            terminado = true;
            iniciado = false;
        }
        else if(tablero.posicionesLibres <= 0){
            terminado = true;
            iniciado = false;
        }
        else{
            terminado = false;
        }
    }
    
    /**
     * Realiza la logica del juego, coloca fichas, intercambia turnos
     * y comprueba si ya se termino el juego.
     * @param jugador El jugador que va a colocar la ficha
     * @param posicion La posicion de la ficha en el tablero (Entre 1 y 9)
     * @see Juego#colocarFicha(Jugador jugador, int posicion)
     * @see Juego#intercambiarTurnos()
     * @see Juego#comprobar()
     */
    public void logica(Jugador jugador, int posicion){
        int numJugador = jugador.numero;
        
        if(jugadores.get(numJugador-1).turno==true){
            colocarFicha(jugador, posicion);
            intercambiarTurnos();
        }
        comprobar();
    }
    
}
