/*
 * java
 *
 * Created on 13 de octubre de 2006, 02:30 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package clases;

import java.io.Serializable;

/**
 *
 * @author Alejandro Muoz Andrade - almunoz
 */
public class Tablero implements Serializable{
    public int tab[][];
    public int posicionesLibres;
    
    /** Creates a new instance of Tablero */
    public Tablero() {
        tab = new int[3][3];
        posicionesLibres = 9;
        
        for(int fila=0; fila<3; fila++){
            for(int columna=0; columna<3; columna++){
                tab[fila][columna] = 0;
            }
        }
    }
    
    /**
     * Retorna la fila en la que esta la posicion dada
     * @param pos La posicion en el tablero (Entre 1 y 9)
     * @return La fila en la que se encuentra la posicion (Entre 0 y 2).
     * Si la posicion no es valida retorna -1 
     */
    public int getFila(int pos){
        if(pos==1 || pos==2 || pos==3)
            return 0;
        else if(pos==4 || pos==5 || pos==6)
            return 1;
        else if(pos==7 || pos==8 || pos==9)
            return 2;
        else
            return -1;
    }
    
    /**
     * Retorna la columna en la que esta la posicion dada
     * @param pos La posicion en el tablero (Entre 1 y 9)
     * @return La columna en la que se encuentra la posicion (Entre 0 y 2).
     * Si la posicion no es valida retorna -1 
     */
    public int getColumna(int pos){
        if(pos==1 || pos==4 || pos==7)
            return 0;
        else if(pos==2 || pos==5 || pos==8)
            return 1;
        else if(pos==3 || pos==6 || pos==9)
            return 2;
        else
            return -1;
    }
    
    /**
     * Comprueba si la posicin dada est vacia.
     * @param posicion La posicion que va ser comprobada en el tablero de juego.
     * Debe ser un nmero entre 1 y 9 
     * @return <code>true</code> - Si la posicin est vacia.
     * @return <code>false</code> - Si la posicin est ocupada o no es una posicin vlida.
     */
    public boolean posicionVacia(int posicion){
        int fila = getFila(posicion);
        int columna = getColumna(posicion);
        
        if(fila==-1 || columna==-1)
            return false;
        if(tab[fila][columna]==0)
            return true;
        else
            return false;
    }
    
    /**
     * Reinicia el tablero a sus valores iniciales
     */
    public void reiniciar(){
        posicionesLibres = 9;
        for(int fila=0; fila<3; fila++){
            for(int columna=0; columna<3; columna++){
                tab[fila][columna] = 0;
            }
        }
    }
    
    /**
     * Coloca una ficha en el tablero de juego
     * @param jugador El jugador que va a colocar la ficha
     * @param posicion La posicion de la ficha en el tablero (Entre 1 y 9)
     */
    public void colocarFicha(Jugador jugador, int posicion){
        int fila = getFila(posicion);
        int columna = getColumna(posicion);
        
        tab[fila][columna] = jugador.numero;
        posicionesLibres = posicionesLibres - 1;
    }
    
    /**
     * Retorna el numero del jugador en la posicion dada
     * @param posicion La posicion del tablero que se quiere verificar
     * @return El numero del jugador en la posicion dada
     */
    public int getNumJugador(int posicion){
        int fila = getFila(posicion);
        int columna = getColumna(posicion);
        
        return tab[fila][columna];
    }
    
    /**
     * Conprueba si el juego se termino
     * @param num El numero del jugador para hacer la comprobacion (1 o 2) 
     * @return <code>true</code> - Si el juego se termino.
     * @return <code>false</code> - Si el juego no se ha terminado.
     */
    public boolean comprobarTerminado(int num){
        //* Comprueba las lineas horizontales */
        if(tab[0][0] == num && tab[0][0] == tab[0][1] && tab[0][0] == tab[0][2])
            return true;
        else if(tab[1][0] == num && tab[1][0] == tab[1][1] && tab[1][0] == tab[1][2])
            return true;
        else if(tab[2][0] == num && tab[2][0] == tab[2][1] && tab[2][0] == tab[2][2])
            return true;
        //* Comprueba las lineas verticales */
        else if(tab[0][0] == num && tab[0][0] == tab[1][0] && tab[0][0] == tab[2][0])
            return true;
        else if(tab[0][1] == num && tab[0][1] == tab[1][1] && tab[0][1] == tab[2][1])
            return true;
        else if(tab[0][2] == num && tab[0][2] == tab[1][2] && tab[0][2] == tab[2][2])
            return true;
        //* Comprueba las lineas diagonales */
        else if(tab[0][0] == num && tab[0][0] == tab[1][1] && tab[0][0] == tab[2][2])
            return true;
        else if(tab[2][0] == num && tab[2][0] == tab[1][1] && tab[2][0] == tab[0][2])
            return true;
        else
            return false;
    }
    
}
