/*
 * TriquiGUI.java
 *
 * Created on 14 de octubre de 2006, 02:55 PM
 */

package cliente;

import clases.*;
import servidor.*;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.server.UnicastRemoteObject;
import java.awt.event.*;
import java.rmi.RemoteException;
import javax.swing.*;

/**
 * Esta clase controla la interfaz grafica del juego y los eventos
 * asociados a ella.<br>
 * Tambin provee los mtodos necesarios para realizar la comunicacin con
 * el servidor remoto, encargado de administrar el juego, adems de la
 * implementacin de los servicios 'callback' que el cliente presta al servidor. 
 *
 * @author Alejandro Muoz Andrade - almunoz
 */
public class TriquiGUI extends javax.swing.JFrame implements IClienteTriqui, Serializable {
    /**
     * Objetos para manejar algunas funciones locales
     * y realizar alguna logica en el cliente.
     */
    private Juego juego;
    private Jugador jugador;
    private IServidorTriqui stubServidor;
    /**
     * Tiempo de espera del reloj para ejecutar la funcin de incremento
     * de la progressBar
     */
    private final static int TIEMPO = 1000;
    private javax.swing.Timer timer;
    private int contProgress;
    
    /**
     * Crea una nueva TriquiGUI para el juego.
     * @exception <code>RemoteException</code> - Si falla la exportacin del objeto
     */
    public TriquiGUI() {
        JDialog.setDefaultLookAndFeelDecorated(true);
        initComponents();
        imagenesMenu.setVisible(false);
        /** Para mostrar una progressBar de espera */
        timer = new Timer(TIEMPO, new TimerListener());
        contProgress = 1;
        esperaProgressBar.setVisible(false);
        juego = new Juego();
        jugador = new Jugador();
    }
    
    /**
     * Ejecuta la marca de reloj especificada en el objeto timer.
     * Este tiempo es el valor de especificado en la variable TIEMPO, dada en milisegundos
     */
    class TimerListener implements ActionListener {
        public void actionPerformed(ActionEvent evt) {
            esperaProgressBar.setValue(contProgress);
            if (contProgress==esperaProgressBar.getMaximum()+1) {
                contProgress = 1;
            }
            contProgress++;
        }
    }
    
    /**
     * This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jPanel1 = new javax.swing.JPanel();
        tableroPanel = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jButton3 = new javax.swing.JButton();
        jButton4 = new javax.swing.JButton();
        jButton5 = new javax.swing.JButton();
        jButton6 = new javax.swing.JButton();
        jButton7 = new javax.swing.JButton();
        jButton8 = new javax.swing.JButton();
        jButton9 = new javax.swing.JButton();
        infoScrollPane = new javax.swing.JScrollPane();
        infoTextArea = new javax.swing.JTextArea();
        jPanel2 = new javax.swing.JPanel();
        esperaProgressBar = new javax.swing.JProgressBar();
        MenuBar = new javax.swing.JMenuBar();
        juegoMenu = new javax.swing.JMenu();
        nuevoMenuItem = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        salirMenuItem = new javax.swing.JMenuItem();
        ayudaMenu = new javax.swing.JMenu();
        contenidoMenuItem = new javax.swing.JMenuItem();
        acercaDeMenuItem = new javax.swing.JMenuItem();
        imagenesMenu = new javax.swing.JMenu();
        equisMenuItem = new javax.swing.JMenuItem();
        circuloMenuItem = new javax.swing.JMenuItem();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Triqui Distribuido - RMI");
        setResizable(false);
        jPanel1.setLayout(new java.awt.GridBagLayout());

        jPanel1.setMinimumSize(new java.awt.Dimension(414, 210));
        jPanel1.setPreferredSize(new java.awt.Dimension(414, 210));
        tableroPanel.setLayout(new java.awt.GridBagLayout());

        tableroPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Tablero de Juego"));
        tableroPanel.setMinimumSize(new java.awt.Dimension(190, 190));
        tableroPanel.setPreferredSize(new java.awt.Dimension(190, 190));
        jButton1.setPreferredSize(new java.awt.Dimension(48, 48));
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 3, 3, 3);
        tableroPanel.add(jButton1, gridBagConstraints);

        jButton2.setPreferredSize(new java.awt.Dimension(48, 48));
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 3, 3, 3);
        tableroPanel.add(jButton2, gridBagConstraints);

        jButton3.setPreferredSize(new java.awt.Dimension(48, 48));
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 3, 3, 3);
        tableroPanel.add(jButton3, gridBagConstraints);

        jButton4.setPreferredSize(new java.awt.Dimension(48, 48));
        jButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton4ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        tableroPanel.add(jButton4, gridBagConstraints);

        jButton5.setPreferredSize(new java.awt.Dimension(48, 48));
        jButton5.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton5ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        tableroPanel.add(jButton5, gridBagConstraints);

        jButton6.setPreferredSize(new java.awt.Dimension(48, 48));
        jButton6.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton6ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        tableroPanel.add(jButton6, gridBagConstraints);

        jButton7.setPreferredSize(new java.awt.Dimension(48, 48));
        jButton7.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton7ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        tableroPanel.add(jButton7, gridBagConstraints);

        jButton8.setPreferredSize(new java.awt.Dimension(48, 48));
        jButton8.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton8ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        tableroPanel.add(jButton8, gridBagConstraints);

        jButton9.setPreferredSize(new java.awt.Dimension(48, 48));
        jButton9.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton9ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        tableroPanel.add(jButton9, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        jPanel1.add(tableroPanel, gridBagConstraints);

        infoScrollPane.setBorder(javax.swing.BorderFactory.createTitledBorder("Informaci\u00f3n"));
        infoScrollPane.setMinimumSize(new java.awt.Dimension(190, 190));
        infoScrollPane.setPreferredSize(new java.awt.Dimension(190, 190));
        infoTextArea.setBackground(infoScrollPane.getBackground());
        infoTextArea.setColumns(14);
        infoTextArea.setEditable(false);
        infoTextArea.setFont(new java.awt.Font("Tahoma", 1, 11));
        infoTextArea.setRows(5);
        infoTextArea.setTabSize(4);
        infoScrollPane.setViewportView(infoTextArea);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        jPanel1.add(infoScrollPane, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 0, 3);
        getContentPane().add(jPanel1, gridBagConstraints);

        jPanel2.setLayout(new java.awt.GridBagLayout());

        jPanel2.setMinimumSize(new java.awt.Dimension(414, 30));
        jPanel2.setPreferredSize(new java.awt.Dimension(414, 30));
        esperaProgressBar.setMinimumSize(new java.awt.Dimension(380, 18));
        esperaProgressBar.setPreferredSize(new java.awt.Dimension(385, 18));
        esperaProgressBar.setString("Esperando al otro jugador...");
        esperaProgressBar.setStringPainted(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.ipady = 3;
        gridBagConstraints.insets = new java.awt.Insets(0, 2, 6, 0);
        jPanel2.add(esperaProgressBar, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(0, 3, 3, 3);
        getContentPane().add(jPanel2, gridBagConstraints);

        juegoMenu.setText("Juego");
        nuevoMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F2, 0));
        nuevoMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/cliente/images/new.png")));
        nuevoMenuItem.setText("Nuevo juego");
        nuevoMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nuevoMenuItemActionPerformed(evt);
            }
        });

        juegoMenu.add(nuevoMenuItem);

        juegoMenu.add(jSeparator1);

        salirMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/cliente/images/exit.png")));
        salirMenuItem.setText("Salir");
        salirMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                salirMenuItemActionPerformed(evt);
            }
        });

        juegoMenu.add(salirMenuItem);

        MenuBar.add(juegoMenu);

        ayudaMenu.setText("Ayuda");
        contenidoMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F1, 0));
        contenidoMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/cliente/images/help.png")));
        contenidoMenuItem.setText("Contenido");
        contenidoMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                contenidoMenuItemActionPerformed(evt);
            }
        });

        ayudaMenu.add(contenidoMenuItem);

        acercaDeMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F5, 0));
        acercaDeMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/cliente/images/about.png")));
        acercaDeMenuItem.setText("Acerca de ...");
        acercaDeMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                acercaDeMenuItemActionPerformed(evt);
            }
        });

        ayudaMenu.add(acercaDeMenuItem);

        MenuBar.add(ayudaMenu);

        imagenesMenu.setText("Imagenes");
        equisMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/cliente/images/equis.png")));
        equisMenuItem.setText("Jugador 1");
        imagenesMenu.add(equisMenuItem);

        circuloMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/cliente/images/circulo.png")));
        circuloMenuItem.setText("Jugador 2");
        imagenesMenu.add(circuloMenuItem);

        MenuBar.add(imagenesMenu);

        setJMenuBar(MenuBar);

        pack();
    }// </editor-fold>//GEN-END:initComponents
    
    private void nuevoMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nuevoMenuItemActionPerformed
        nuevoAction();  
    }//GEN-LAST:event_nuevoMenuItemActionPerformed

    private void jButton9ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton9ActionPerformed
        ButtonAction(9);  
    }//GEN-LAST:event_jButton9ActionPerformed

    private void jButton8ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton8ActionPerformed
        ButtonAction(8);
    }//GEN-LAST:event_jButton8ActionPerformed

    private void jButton7ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton7ActionPerformed
        ButtonAction(7);
    }//GEN-LAST:event_jButton7ActionPerformed

    private void jButton6ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton6ActionPerformed
        ButtonAction(6);
    }//GEN-LAST:event_jButton6ActionPerformed

    private void jButton5ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton5ActionPerformed
        ButtonAction(5);
    }//GEN-LAST:event_jButton5ActionPerformed

    private void jButton4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton4ActionPerformed
        ButtonAction(4);
    }//GEN-LAST:event_jButton4ActionPerformed

    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
        ButtonAction(3);
    }//GEN-LAST:event_jButton3ActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        ButtonAction(2);
    }//GEN-LAST:event_jButton2ActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        ButtonAction(1);
    }//GEN-LAST:event_jButton1ActionPerformed

    private void salirMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_salirMenuItemActionPerformed
        salirAction();
    }//GEN-LAST:event_salirMenuItemActionPerformed

    private void acercaDeMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_acercaDeMenuItemActionPerformed
        acercaDeDialogo();
    }//GEN-LAST:event_acercaDeMenuItemActionPerformed

    private void contenidoMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_contenidoMenuItemActionPerformed
        mensajeDialogo("La Ayuda no se encuentra disponible \n" +
                       "desde la aplicacin por el momento.");
    }//GEN-LAST:event_contenidoMenuItemActionPerformed
    
    //* FUNCIONES PARA GUI */
    private void nuevoAction(){
        try{
            int nuevo = confirmacionDialogo("Est seguro que desea iniciar un juego nuevo?");
            if(nuevo==JOptionPane.OK_OPTION){
                if(stubServidor!=null)
                    stubServidor.nuevoJuego(jugador);
                juego.reiniciar();
                reiniciarGUI();
                conectar();
            }
        }
        catch(Exception e){
            errorDialogo("No se pudo conectar con el servidor.\nIntentelo mas tarde por favor.");
        }
    }
    
    private void salirAction(){
        try{
            int salir = salirDialogo();
            if(salir==JOptionPane.OK_OPTION){
                if(stubServidor!=null){
                    stubServidor.nuevoJuego(jugador);
                }
                System.exit(0);
            }
        }
        catch(Exception e){
            errorDialogo("No se pudo conectar con el servidor.\nIntentelo mas tarde por favor.");
            System.exit(0);
        }
    }
    
    private void ButtonAction(int posicion){
        jugador = jugar(jugador, posicion);
    }
    
    private void iniciarProgressBar(){
        contProgress = 1;
        esperaProgressBar.setValue(0);
        esperaProgressBar.setVisible(true);
        if(!timer.isRunning()){
            timer.start();
        }
    }
    
    private void detenerProgressBar(){
        if(timer.isRunning()){
            timer.stop();
        }
        esperaProgressBar.setVisible(false);
        esperaProgressBar.setValue(0);
        contProgress = 1;
    }
    
    private void acercaDeDialogo(){
        String mensaje = "Triqui Distribuido - RMI \n" +
                         "Version 1.0 \n" +   
                         "27 de Octubre de 2006 \n\n" +
                         "Alejandro Muoz Andrade \n" +
                         "Estudiante Ingenieria de Sistemas \n" +
                         "Universidad del Cauca \n" +
                         "Copyright \u00A9 2006 \n\n";
        String titulo = "Triqui Distribuido - Acerca de...";
        JOptionPane.showMessageDialog(this , mensaje, titulo, JOptionPane.INFORMATION_MESSAGE);
    }
    
    private void mensajeDialogo(String mensaje){
        String titulo = "Triqui Distribuido - Mensaje de Informacin";
        JOptionPane.showMessageDialog(this , mensaje, titulo, JOptionPane.INFORMATION_MESSAGE);
    }
    
    private int confirmacionDialogo(String mensaje){
        String titulo = "Triqui Distribuido - Mensaje de Confirmacin";
        int opcion = JOptionPane.showConfirmDialog(this, mensaje, titulo,
                JOptionPane.OK_CANCEL_OPTION,
                JOptionPane.QUESTION_MESSAGE);
        return opcion;
    }
    
    private void errorDialogo(String mensaje){
        String titulo = "Triqui Distribuido - Mensaje de Error";
        JOptionPane.showMessageDialog(this , mensaje, titulo, JOptionPane.ERROR_MESSAGE);
    }
    
    private void warningDialogo(String mensaje){
        String titulo = "Triqui Distribuido - Mensaje de Advertencia";
        JOptionPane.showMessageDialog(this , mensaje, titulo, JOptionPane.WARNING_MESSAGE);
    }
    
    private int salirDialogo() {
        String mensaje = "Confirma que Desea Salir?";
        String titulo = "Triqui Distribuido";
        int opcion = JOptionPane.showConfirmDialog(this, mensaje, titulo,
                JOptionPane.OK_CANCEL_OPTION,
                JOptionPane.QUESTION_MESSAGE);
        return opcion;
    }
    
    private void nuevoJuegoRun(String mensaje){
        final String mensajeRun = mensaje;
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run(){
                mensajeDialogo(mensajeRun);
                juego.reiniciar();
                reiniciarGUI();
                conectar();
            }
        });
    }
    
    private void terminarDialogoRun(String mensaje){
        final String mensajeRun = mensaje;
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run(){
                    mensajeDialogo(mensajeRun);
                    juego.reiniciar();
                    reiniciarGUI();
                    conectar();
            }
        });
    }
    
    private void mensajeDialogoRun(String mensaje){
        final String mensajeRun = mensaje;
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run(){
                mensajeDialogo(mensajeRun);
            }
        });
    }
    
    private void mensajeText(String mensaje){
        infoTextArea.setText(infoTextArea.getText() + mensaje + "\n");
    }
    
    private void reiniciarGUI(){
        jButton1.setIcon(null);
        jButton2.setIcon(null);
        jButton3.setIcon(null);
        jButton4.setIcon(null);
        jButton5.setIcon(null);
        jButton6.setIcon(null);
        jButton7.setIcon(null);
        jButton8.setIcon(null);
        jButton9.setIcon(null);
        infoTextArea.setText("");
        detenerProgressBar();
    }
    
    private void actualizarTableroGUI(Tablero tablero){
        int posicion = 1;
        Jugador jugadorTemp = new Jugador();
        
        for(int fila=0; fila<3; fila++){
            for(int columna=0; columna<3; columna++){
                if(tablero.tab[fila][columna]==1){
                    jugadorTemp.numero = 1;
                }
                else if(tablero.tab[fila][columna]==2){
                    jugadorTemp.numero = 2;
                }
                else{
                    jugadorTemp.numero = 0;
                }
                colocarFichaGUI(jugadorTemp, posicion);
                posicion++;
            }
        }
        
    }
    
    private void colocarFichaGUI(Jugador jugador, int posicion){
        switch(posicion){
            case 1:
                    if(jugador.numero==1)
                        jButton1.setIcon(equisMenuItem.getIcon());
                    else if(jugador.numero==2)
                        jButton1.setIcon(circuloMenuItem.getIcon());
                    else
                        jButton1.setIcon(null);
                    break;
            case 2:
                    if(jugador.numero==1)
                        jButton2.setIcon(equisMenuItem.getIcon());
                    else if(jugador.numero==2)
                        jButton2.setIcon(circuloMenuItem.getIcon());
                    else
                        jButton2.setIcon(null);
                    break;
            case 3:
                    if(jugador.numero==1)
                        jButton3.setIcon(equisMenuItem.getIcon());
                    else if(jugador.numero==2)
                        jButton3.setIcon(circuloMenuItem.getIcon());
                    else
                        jButton3.setIcon(null);
                    break;
            case 4:
                    if(jugador.numero==1)
                        jButton4.setIcon(equisMenuItem.getIcon());
                    else if(jugador.numero==2)
                        jButton4.setIcon(circuloMenuItem.getIcon());
                    else
                        jButton4.setIcon(null);
                    break;
            case 5:
                    if(jugador.numero==1)
                        jButton5.setIcon(equisMenuItem.getIcon());
                    else if(jugador.numero==2)
                        jButton5.setIcon(circuloMenuItem.getIcon());
                    else
                        jButton5.setIcon(null);
                    break;
            case 6:
                    if(jugador.numero==1)
                        jButton6.setIcon(equisMenuItem.getIcon());
                    else if(jugador.numero==2)
                        jButton6.setIcon(circuloMenuItem.getIcon());
                    else
                        jButton6.setIcon(null);
                    break;
            case 7:
                    if(jugador.numero==1)
                        jButton7.setIcon(equisMenuItem.getIcon());
                    else if(jugador.numero==2)
                        jButton7.setIcon(circuloMenuItem.getIcon());
                    else
                        jButton7.setIcon(null);
                    break;
            case 8:
                    if(jugador.numero==1)
                        jButton8.setIcon(equisMenuItem.getIcon());
                    else if(jugador.numero==2)
                        jButton8.setIcon(circuloMenuItem.getIcon());
                    else
                        jButton8.setIcon(null);
                    break;
            case 9:
                    if(jugador.numero==1)
                        jButton9.setIcon(equisMenuItem.getIcon());
                    else if(jugador.numero==2)
                        jButton9.setIcon(circuloMenuItem.getIcon());
                    else
                        jButton9.setIcon(null);
                    break;
            default:
                    break;
        }
        
    }
    
    //* FUNCIONES PARA LA LOGICA DEL JUEGO */
    private boolean conectar(){
        try{
            jugador = stubServidor.conexion(this); // Servicio
            if (jugador == null) {
                mensajeDialogo("Por el momento no hay juegos disponibles.\n" +
                               "Intentelo mas tarde por favor.");
                return false;
            }
            
            int numJugador = jugador.numero;
            int numJuego = jugador.numeroJuego;
            juego.numero = jugador.numeroJuego;
            infoScrollPane.setBorder(BorderFactory.createTitledBorder("Informaci\u00f3n: Jugador " + numJugador + " (" + jugador.ficha + ")"));
            mensajeText("Juego nmero: " + (numJuego+1) + "\n");
            if(jugador.turno){
                mensajeText("Elija una posicin por favor.");
            } else if(!jugador.turno){
                mensajeText("Espere su turno por favor.");
                iniciarProgressBar();
            }
            return true;
        }
        catch(Exception e){
            errorDialogo("No se pudo conectar con el servidor.\nIntentelo mas tarde por favor.");
            return false;
        }
    }
    
    private Jugador jugar(Jugador jugador, int posicion){
        try{
            int numJuego = jugador.numeroJuego;
            int numJugador = jugador.numero;
            
            if(jugador.turno){
                if(!juego.posicionVacia(posicion)){
                    errorDialogo("Esta posicin esta ocupada.\nSeleccione otra por favor.");
                    return jugador;
                }
                colocarFichaGUI(jugador, posicion);
                juego = stubServidor.logica(jugador, posicion);
                jugador = juego.jugadores.get(jugador.numero-1);
            }
            else if(!jugador.turno){
                errorDialogo("Espere su turno por favor.");
            }
            
            return jugador;
        }
        catch(Exception e){
            errorDialogo("No se pudo conectar con el servidor.\nIntentelo mas tarde por favor.");
            return jugador;
        }
    }
    
    // RUTINAS DE SERVICIO PARA EL CALLBACK
    public void cbActualizarCliente(Juego game) throws RemoteException{
        juego = game;
        int numJuego = juego.numero;
        String mensaje = "Callback";
        jugador = juego.jugadores.get(jugador.numero-1);
        
        actualizarTableroGUI(juego.tablero);
        if(jugador.turno){
            detenerProgressBar();
            if(juego.terminado){
                if(juego.tablero.posicionesLibres<=0){
                    mensaje = "Juego Terminado en Empate.";
                } else{
                    mensaje = "Usted ha perdido.\nJuego Terminado.";
                }
                mensajeText(mensaje);
                terminarDialogoRun(mensaje);
            }
            else{
                mensajeText("Elija una posicin por favor.");
            }
        }
        else if(!jugador.turno){
            iniciarProgressBar();
            if(juego.terminado){
                if(juego.tablero.posicionesLibres<=0){
                    mensaje = "Juego Terminado en Empate.";
                } else{
                    mensaje = "Usted ha ganado.\nJuego Terminado.";
                }
                mensajeText(mensaje);
                terminarDialogoRun(mensaje);
            }
            else{
                mensajeText("Espere su turno por favor.");
            }
        }
    }
    
    public void cbActualizarTablero(Juego game) throws RemoteException{
        juego = game;
        actualizarTableroGUI(juego.tablero);
    }
    
    public void cbMensajeText(String mensaje) throws RemoteException{
        mensajeText(mensaje);
    }
    
    public void cbMensajeDialogo(String mensaje) throws RemoteException{
        mensajeDialogoRun(mensaje);
    }
    
    public void cbNuevoJuego() throws RemoteException{
        nuevoJuegoRun("El otro jugador se ha retirado.\nSe iniciara un juego nuevo.");
    }
    
    public void iniciarRMI(String host, String puerto){
        try{
            String name = "rmi://"+host+":"+puerto+"/Triqui";
            //stubServidor = new ServidorTriqui();
            stubServidor = (IServidorTriqui) Naming.lookup(name);
            UnicastRemoteObject.exportObject(this);
        }
        catch (Exception e){
            errorDialogo("No se pudo conectar con el servidor.\nIntentelo mas tarde por favor.");
        }
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]){
        String host = "localhost";
        String puerto =  "1099";
        if(args.length==1){
            host = args[0];
        }
        else if(args.length==2){
            host = args[0];
            puerto = args[1];
        }
        final String hostf = host;
        final String puertof =  puerto;
        setDefaultLookAndFeelDecorated(true);
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run(){
                TriquiGUI triquiGUI = new TriquiGUI();
                triquiGUI.setVisible(true);
                triquiGUI.iniciarRMI(hostf, puertof);
                if(triquiGUI.stubServidor!=null)
                    triquiGUI.conectar();
                else
                    System.exit(0);
            }
        });
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuBar MenuBar;
    private javax.swing.JMenuItem acercaDeMenuItem;
    private javax.swing.JMenu ayudaMenu;
    private javax.swing.JMenuItem circuloMenuItem;
    private javax.swing.JMenuItem contenidoMenuItem;
    private javax.swing.JMenuItem equisMenuItem;
    private javax.swing.JProgressBar esperaProgressBar;
    private javax.swing.JMenu imagenesMenu;
    private javax.swing.JScrollPane infoScrollPane;
    private javax.swing.JTextArea infoTextArea;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton jButton4;
    private javax.swing.JButton jButton5;
    private javax.swing.JButton jButton6;
    private javax.swing.JButton jButton7;
    private javax.swing.JButton jButton8;
    private javax.swing.JButton jButton9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JMenu juegoMenu;
    private javax.swing.JMenuItem nuevoMenuItem;
    private javax.swing.JMenuItem salirMenuItem;
    private javax.swing.JPanel tableroPanel;
    // End of variables declaration//GEN-END:variables
    
}
