/*
 * ServidorTriqui.java
 *
 * Created on 14 de octubre de 2006, 11:56 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package servidor;

import clases.*; 
import cliente.IClienteTriqui;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.registry.LocateRegistry;
import java.util.Vector;
import java.rmi.Naming;    
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.rmi.server.UnicastRemoteObject;

/**
 * Esta clase implementa los servicios ofrecidos por el servidor.
 * @author Alejandro Muoz Andrade - almunoz
 */
public class ServidorTriqui extends UnicastRemoteObject implements IServidorTriqui, Serializable{
    public static final int MAX_JUEGOS = 10;
    public static final int MAX_CLIENTES = MAX_JUEGOS*2;
    public static Vector<Juego> juegos;
    public static Vector<IClienteTriqui> clientes; 
    
    /** Creates a new instance of ServidorTriqui */
    public ServidorTriqui() throws RemoteException {
        juegos = new Vector<Juego>(10, 1);
        clientes = new Vector<IClienteTriqui>(20, 1);
        for(int index=0; index<MAX_JUEGOS; index++){
            juegos.add(new Juego(index));
        }
        for(int i=0; i<MAX_CLIENTES; i++){
            clientes.add(null);
        }
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        try{
            String host = "localhost";
            String puerto =  "1099";
            if(args.length==1){
                host = args[0];
            }
            else if(args.length==2){
                host = args[0];
                puerto = args[1];
            }
            String name = "rmi://"+host+":"+puerto+"/Triqui";
            IServidorTriqui stubServidor = new ServidorTriqui();
            /*if (System.getSecurityManager() == null) {
                System.setSecurityManager(new RMISecurityManager()); 			
            }*/
            LocateRegistry.createRegistry(Integer.parseInt(puerto));
            Naming.rebind(name, stubServidor);
            System.out.println("Servidor Triqui Iniciado: " + name);
        }
        catch (RemoteException e) {
            System.err.println("Servidor Triqui Exception: " +  e.getMessage());
        }
        catch (MalformedURLException e){
            System.err.println("Servidor Triqui Exception: " +  e.getMessage());
        }
    }
    
    /**
     * Inicializa la lista de juegos que van a estar disponibles en la aplicacion
     */
    public void iniciarListaJuegos(){
        if(juegos.isEmpty()){
            for(int index=0; index<MAX_JUEGOS; index++){
                juegos.add(new Juego(index));
            }
        }
    }
    
    /**
     * Verifica si hay un juego disponible
     */
    public boolean juegoDisponible(){
        for(int index=0; index<MAX_JUEGOS; index++){
            if(juegos.get(index).estaDisponible()){
                return true;
            }
        }
        return false;
    }
    
    /**
     * Verifica si hay un juego en espera
     */
    public boolean juegoEsperando(){
        for(int index=0; index<MAX_JUEGOS; index++){
            if(juegos.get(index).estaEsperando()){
                return true;
            }
        }
        return false;
    }
    
    /**
     * Busca un juego que ya se haya iniciado y devuelve el jugador que hace falta
     */
    public Jugador unirseJuego(){
        for(int index=0; index<MAX_JUEGOS; index++){
            if(juegos.get(index).estaEsperando()){
                return juegos.get(index).unirse();
            }
        }
        return null;
    }
    
    /**
     * Inicia un juego nuevo y devuelve el jugador asignado
     */
    public Jugador iniciarJuegoNuevo(){
        for(int index=0; index<MAX_JUEGOS; index++){
            if(juegos.get(index).estaDisponible()){
                juegos.get(index).reiniciar();
                return juegos.get(index).iniciarNuevo();
            }
        }
        return null;
    }
    
    /**
     * Actualiza los clientes mediante un callback a ellos
     */
    public void actualizarClientes(Juego juego){
        int numCliente1 = calcularNumCliente(juego.jugadores.get(0));
        int numCliente2 = calcularNumCliente(juego.jugadores.get(1));
        
        try{
            if(clientes.get(numCliente1)!=null){
                clientes.get(numCliente1).cbActualizarCliente(juego);
            }
            if(clientes.get(numCliente2)!=null){
                clientes.get(numCliente2).cbActualizarCliente(juego);
            }
        }
        catch(Exception e){
            System.out.println("Callback Exception: " + e.getMessage());
        }
    }
    
    /**
     * Calcula el numero del cliente de acuerdo al numero del juego y al
     * numero de jugador, todo esto se hace para poder realizar el callback
     */
    public int calcularNumCliente(Jugador jugador){
        int numCliente = (2*jugador.numeroJuego) + (jugador.numero - 1);
        return numCliente;
    }
    
    /**
     * Inicia un juego nuevo
     */
    public void nuevoJuego(Jugador jugador){
        int numJuego = jugador.numeroJuego;
        int numCliente1 = calcularNumCliente(juegos.get(numJuego).jugadores.get(0));
        int numCliente2 = calcularNumCliente(juegos.get(numJuego).jugadores.get(1));
        try{
            juegos.get(numJuego).reiniciar();
            if(jugador.numero==1){
                clientes.set(numCliente1, null);
                System.out.println("Juego " + (jugador.numeroJuego+1) + ": Jugador 1 desconectado.");
                if(clientes.get(numCliente2)!=null){
                    clientes.get(numCliente2).cbNuevoJuego();
                    System.out.println("Juego " + (jugador.numeroJuego+1) + ": Jugador 2 desconectado.");
                }
            }
            else if(jugador.numero==2){
                clientes.set(numCliente2, null);
                System.out.println("Juego " + (jugador.numeroJuego+1) + ": Jugador 2 desconectado.");
                if(clientes.get(numCliente1)!=null){
                    clientes.get(numCliente1).cbNuevoJuego();
                    System.out.println("Juego " + (jugador.numeroJuego+1) + ": Jugador 1 desconectado.");
                }
            }
        }
        catch (RemoteException e){
            System.out.println(e.getMessage());
        }
    }
    
    /**
     * Reinicia un juego
     */
     public void reiniciarJuego(int numJuego){
        juegos.get(numJuego).reiniciar();
    }
     
    /**
     * Verifica si hay juegos disponibles y retorna el jugador asigando
     */
    public Jugador conexion(IClienteTriqui cliente){
        Jugador jugador = new Jugador();
                
        if(juegoDisponible()){
            try{
                if (juegoEsperando()){
                    jugador = unirseJuego();
                    System.out.println("Juego " + (jugador.numeroJuego+1) + ": Jugador " + jugador.numero + " conectado.");
                    cliente.cbMensajeText("Uniendose a un juego.");
                    cliente.cbActualizarTablero(juegos.get(jugador.numeroJuego));
                }
                else{
                    jugador = iniciarJuegoNuevo();
                    System.out.println("Juego " + (jugador.numeroJuego+1) + ": Jugador " + jugador.numero + " conectado.");
                    cliente.cbMensajeText("Iniciando un nuevo juego.");
                }
            }
            catch (RemoteException e){
                System.out.println("Error en el callback al cliente: " + e.getMessage());
            }
        }
        else{
            jugador = null;
        }
        
        if(jugador!=null){
            int numCliente = calcularNumCliente(jugador);
            clientes.set(numCliente, cliente);
        }
        return jugador; 
    }
    
    /**
     * Realiza la logica del juego llamando al metodo especificado en
     * la clase Juego, tambien hace el llamado al metodo necesario para
     * realizar el callback a los clientes, y reinicia el juego si se termino.
     * Retorna un objeto Juego actualizado.
     */
    public Juego logica(Jugador jugador, int posicion) throws RemoteException{
        int numJuego = jugador.numeroJuego;
                
        juegos.get(numJuego).logica(jugador, posicion);
        actualizarClientes(juegos.get(numJuego));
        if(juegos.get(numJuego).terminado){
            reiniciarJuego(numJuego);
            System.out.println("Juego " + (numJuego+1) + ": Terminado.");
        }
        return juegos.get(numJuego);
    }
    
}
